#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "openssl/hmac.h"

static const char HIDDEN_PATH[] = "/dist/";
static const char SECRET_KEY[]  = "secret key"; 
static const int SIG_LENGTH = SHA_DIGEST_LENGTH * 2;


static table *parse_cookie(request_rec *r)
{
    const char *raw_cookie, *token, *key, *value;
    table *cookie;

    cookie = ap_make_table(r->pool, 5);
    raw_cookie = ap_table_get(r->headers_in, "Cookie");
    if (raw_cookie == NULL)
        return cookie;

    while (*raw_cookie && (token = ap_getword(r->pool, &raw_cookie, ' '))) {
         char *semi;

	 key   = ap_getword(r->pool, &token, '=');
         value = ap_getword(r->pool, &token, '=');
         if ((semi = strrchr(value, ';')) != NULL)
             *semi = '\0';

         ap_table_set(cookie, key, value);
    }
    return cookie;
}


static char *compute_signature(pool *p, const char *input, const char *key)
{
 //   return (char *)ap_md5(p, ap_pstrcat(p, input, key, NULL));
    unsigned char hmac[EVP_MAX_MD_SIZE];
    unsigned char *rc;
    unsigned int hmac_len, i;
    char hex_hmac[SHA_DIGEST_LENGTH * 2], *h;
    const char hex[] = "0123456789abcdef";

    rc = HMAC(EVP_sha1(), key, strlen(key),
              (const unsigned char *)input, strlen(input),
              hmac, &hmac_len);
    if (rc == NULL)
        return NULL;

    h = hex_hmac;
    for (i = 0; i < hmac_len; i++) {
        *h++ = hex[hmac[i] >> 4];
        *h++ = hex[hmac[i] & 0xF];
    }
    *h = '\0';

    return ap_pstrdup(p, hex_hmac);
}


static int is_timeout(pool *p, const char *signed_str)
{
    char *timestr, *embed_signature, *local_signature;
    time_t timeout;

    if (signed_str == NULL || strlen(signed_str) < SIG_LENGTH)
        return 1;
    /* signatureФƷ׻signatureӤ */
    embed_signature = ap_pstrdup(p, &signed_str[strlen(signed_str) - SIG_LENGTH]);
    timestr   = ap_pstrndup(p, signed_str, strlen(signed_str) - SIG_LENGTH);
    /* timeoutͤsignature׻ФsignatureӤ */
    local_signature = compute_signature(p, timestr, SECRET_KEY);
    if (strcmp(local_signature, embed_signature) != 0) {
        return 1;
    }

    timeout = (time_t)atoi(timestr);
    /* ॢȤƤ롩 */
    if (timeout <= time(NULL)) {
        return 1;
    }
    return 0;
}

static int check_valid_access(request_rec *r)
{
    table *cookie;

    /* оݥǥ쥯ȥؤΥ */
    if (strncmp(r->uri, HIDDEN_PATH, strlen(HIDDEN_PATH)) != 0)
        return DECLINED;

    /* »֤᤮Cookie⤷requestξϥ */
    cookie = parse_cookie(r);
    if (is_timeout(r->pool, ap_table_get(cookie, "timeout"))) {
        return FORBIDDEN;
    }
    return OK;
}


/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT access_timeout_hmac_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    NULL,                  /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    check_valid_access,    /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

