#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

static const char AGENT_DOCUMENT_ROOT[] = "/tmp/htdocs";
static const char DEFAULT_AGENT[]       = "default";
static const char HTTP_USER_AGENT[]     = "User-Agent";

/*
 * User-Agent:إåʸ֤
 *   ex:
 *     "Mozilla/4.0 (compatible; MSIE 5.22; Mac_PowerPC)"
 *     -> "Mozilla_4_0"
 * ʹߤڤȤե٥å or ʳ"_"ִ롣
 */
static const char *regularize_user_agent(request_rec *r)
{
    const char *ua;
    char *regularized_ua;
    int len, i;
    
    ua = ap_table_get(r->headers_in, HTTP_USER_AGENT);
    if (ua == NULL)
        return ap_pstrdup(r->pool, DEFAULT_AGENT);

    regularized_ua = ap_getword_white_nc(r->pool, &ua);
    len = strlen(regularized_ua);
    for (i = 0; i < len; i++) {
        if (ap_isalnum(regularized_ua[i]))
            continue;
        regularized_ua[i] = '_';
    }
    return (const char *)regularized_ua;
}

static int translate_agentdir(request_rec *r)
{
    const char *ua;  /* User-Agent name */
    const char *agent_root;
    struct stat dir_stat;

    ua = regularize_user_agent(r);
    agent_root = ap_make_full_path(r->pool, AGENT_DOCUMENT_ROOT, ua);
    if (stat(agent_root, &dir_stat) == -1) {
        agent_root = ap_make_full_path(r->pool,
                                       AGENT_DOCUMENT_ROOT, DEFAULT_AGENT); 
    }
    r->filename = ap_pstrcat(r->pool, agent_root, r->uri, NULL);
    return OK;
}

module MODULE_VAR_EXPORT agentdir_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    NULL,                  /* [#8] MIME-typed-dispatched handlers */
    translate_agentdir,    /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

