#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "http_core.h"
#include "http_log.h"

/* ɽregex˰פ桼̾ʤ鿿 */
static int is_allow_user(request_rec *r, const char *regex)
{
    regex_t *reg;
    
    reg = ap_pregcomp(r->pool, regex, REG_EXTENDED|REG_NOSUB|REG_ICASE);
    if (reg == NULL) {
        ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                      "regex could not be compiled. '%s'", regex);
        return 0;
    }
    return ap_regexec(reg, r->connection->user, 0, NULL, 0) == 0;
}

static int check_user_by_regex(request_rec *r)
{
    const array_header *reqs_arr;
    require_line *reqs;
    int arr_len, i;

    reqs_arr = ap_requires(r);
    if (reqs_arr == NULL)
        return OK;        /* RequireꤵƤʤϵĤ */
    reqs = (require_line *)reqs_arr->elts;
    arr_len = reqs_arr->nelts;
    for (i = 0; i < arr_len; i++) {
        const char *line, *keyword, *regex;

        /* طʤꥯȥ᥽åɤؤ̵뤹 */
        if (!(reqs[i].method_mask & (1 << r->method_number))) {
            continue;
        }

        /* Requireꤵ줿ɤФ */
        line = reqs[i].requirement;
        keyword = ap_getword_white(r->pool, &line);
        if (strcmp(keyword, "user-match") != 0)
            continue;
        /* ɽФ桼̾å */
        regex = ap_getword_conf_nc(r->pool, (char **)&line);
        if (regex != NULL && is_allow_user(r, regex)) {
            return OK;   /*  */
        }
    }
    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                  "access to %s failed, reason: user %s not allowed access",
                  r->uri, r->connection->user);
    ap_note_basic_auth_failure(r);
    return AUTH_REQUIRED;  /*  */
}


/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT auth_regex_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    NULL,                  /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    check_user_by_regex,   /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

