#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "http_log.h"
#include "libapreq/apache_request.h"
#include "openssl/evp.h"
#include "openssl/rand.h"

static const char HTML_FORM[] =
"<html><body>\n"
"  %s<br>\n"
"  <form method=\"POST\">\n"
"    ID: <input type=\"text\" name=\"id\"><br>\n"
"    KEYWORD: <input type=\"password\" name=\"keyword\"><br>\n"
"    MESSAGE: <br><textarea name=\"words\">%s</textarea><br>\n"
"    <input type=\"submit\" name=\"save\" value=\"save\">\n"
"    <input type=\"submit\" name=\"load\" value=\"load\">\n"
"  </form>\n"
"</body></html>";

static const char STORAGE_PATH[] = "/tmp/encrypt_store";
static const char MAGIC[] = "Salted__";

enum {
    MAX_ID_LENGTH = 10,
    ENCRYPT       = 1,
    DECRYPT       = 0,
};



static int is_valid_id(char *src)
{
    char *s;

    if (src == NULL || strlen(src) > MAX_ID_LENGTH)
        return 0;
    for (s = src; *s != '\0'; s++) {
        if (!ap_isalnum(*s))
            return 0;
    }
    return 1;
}


static const char *do_input_check(pool *p, ApacheRequest *req)
{
    if (ApacheRequest_param(req, "save") == NULL
        && ApacheRequest_param(req, "load") == NULL) {
        return "saveޤloadܥ򲡤Ƥ";
    }
    if (ApacheRequest_param(req, "id") == NULL 
        || ApacheRequest_param(req, "keyword") == NULL
        || strlen(ApacheRequest_param(req, "id")) == 0
        || strlen(ApacheRequest_param(req, "keyword")) == 0) {
        return "IDKEYWORDϤƤ";
    }
    if (!is_valid_id((char *)ApacheRequest_param(req, "id"))) {
         return "ID10ʸαѿѤƤ";
    }

    return NULL;
}


/* ʿʸmsgpasswdǰŹ沽롣 */
static unsigned char *encrypt_message(request_rec *r, const unsigned char *msg, 
                           const unsigned char *passwd, int *dist_len)
{
    EVP_CIPHER_CTX ctx;
    EVP_CIPHER *type = EVP_bf_cbc();        /* ѤŹ Blowfish CBC mode */
    unsigned char key[EVP_MAX_KEY_LENGTH];
    unsigned char iv[EVP_MAX_IV_LENGTH];
    unsigned char salt[PKCS5_SALT_LEN];
    time_t t;
    int enc_len, pad_len;
    unsigned char *dist;                    /* ŹʸǼ */

    /* passwdȤsalt, key, IV׻ */
    time(&t);
    RAND_seed((unsigned char *)&t, sizeof(t));
    RAND_pseudo_bytes(salt, sizeof(salt));
    RAND_bytes(iv, sizeof(iv));
    EVP_BytesToKey(type, EVP_md5(), salt, passwd, strlen(passwd), 1, key, iv);

    /* Ź楢르ꥺ */
    EVP_CIPHER_CTX_init(&ctx);
    EVP_CipherInit(&ctx, type, key, iv, ENCRYPT);
    /* ŹʸǼХåե򥢥ȤƬMagic numbersalt */
    dist = ap_palloc(r->pool, strlen(MAGIC) + sizeof(salt)
                              + strlen(msg) + EVP_CIPHER_CTX_block_size(&ctx));
    memmove(dist, MAGIC, strlen(MAGIC));
    memmove(dist + strlen(MAGIC), salt, sizeof(salt));
    *dist_len = strlen(MAGIC) + sizeof(salt);
    /* msgŹ沽 */ 
    EVP_CipherUpdate(&ctx, dist + *dist_len, &enc_len,
                     (unsigned char *)msg, strlen(msg));
    EVP_CipherFinal(&ctx, dist + *dist_len + enc_len, &pad_len);
    EVP_CIPHER_CTX_cleanup(&ctx);
    *dist_len += enc_len + pad_len;

    return dist;
}


/* Źʸsrcpasswd椹 */
static unsigned char *decrypt_message(request_rec *r,
                                      const unsigned char *src, int src_len,
                                      const unsigned char *passwd)
{
    EVP_CIPHER_CTX ctx;
    EVP_CIPHER *type = EVP_bf_cbc();      /* ѤŹ Blowfish CBC mode */
    unsigned char key[EVP_MAX_KEY_LENGTH];
    unsigned char iv[EVP_MAX_IV_LENGTH];
    unsigned char salt[PKCS5_SALT_LEN];
    unsigned char *dist;                  /* ʣ礷åǼ */ 
    int dec_len, pad_len;
    int rc;

    if (src_len < strlen(MAGIC) + sizeof(salt)) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r, "bad decrypt: message is short");
        return NULL;
    }

    /* Magick numberǧ */
    if (strncmp(src, MAGIC, strlen(MAGIC)) != 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r, "bad decrypt: bad magic number");
        return NULL;
    }
    src += strlen(MAGIC);
    /* salt */
    memmove(salt, src, sizeof(salt));
    src += sizeof(salt);
    src_len -= strlen(MAGIC) + sizeof(salt);
    /* passwdȤkey, iv׻ */
    EVP_BytesToKey(type, EVP_md5(), salt,
                   (unsigned char *)passwd, strlen(passwd), 1, key, iv);

    /* Ź楢르ꥺsrc椹 */
    EVP_CIPHER_CTX_init(&ctx);
    EVP_CipherInit(&ctx, type, key, iv, DECRYPT);
    dist = ap_palloc(r->pool, src_len);
    EVP_CipherUpdate(&ctx, dist, &dec_len, (unsigned char *)src, src_len);
    rc = EVP_CipherFinal(&ctx, dist + dec_len, &pad_len);
    EVP_CIPHER_CTX_cleanup(&ctx);
    /* ʥѥɤ */
    if (rc == 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r, "bad decrypt: invalid password?");
        return NULL;
    }
    dist[dec_len + pad_len] = '\0';

    return dist;
}


/* å¸ */
static int save_message(request_rec *r, ApacheRequest *req)
{
    const char *id = ApacheRequest_param(req, "id");
    const unsigned char *key = ApacheRequest_param(req, "keyword");
    const unsigned char *msg = ApacheRequest_param(req, "words");
    char *path;
    int fd;
    const unsigned char *encrypted;
    int enc_len;

    if (id == NULL || key == NULL || msg == NULL)
        return 0;

    /* Ź沽path˽񤭹 */
    encrypted = encrypt_message(r, msg, key, &enc_len);
    path = ap_make_full_path(r->pool, STORAGE_PATH, id);
    fd = ap_popenf(r->pool, path, O_WRONLY|O_CREAT|O_TRUNC, S_IWUSR|S_IRUSR);
    if (fd < 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
                      "cannot open user storage: '%s'", path);
        return 0;
    }
    write(fd, encrypted, enc_len);
    ap_pclosef(r->pool, fd); 
    return 1;
}


/* åɤ߹ */
static char *load_message(request_rec *r, ApacheRequest *req)
{
    const char *id = ApacheRequest_param(req, "id");
    const unsigned char *key = ApacheRequest_param(req, "keyword");
    unsigned char *encrypted, *msg;
    char *path;
    int fd;
    ssize_t msg_r_len;
    struct stat finfo;

    if (id == NULL || key == NULL)
        return NULL;

    /* եpathŹʸencryptedɤ߹ߡ沽 */
    path = ap_make_full_path(r->pool, STORAGE_PATH, id);
    fd = ap_popenf(r->pool, path, O_RDONLY, S_IWUSR|S_IRUSR);
    if (fd < 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
                      "cannot open user strage: '%s'", path);
        return NULL;
    }
    if (fstat(fd, &finfo) != 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
                      "cannot stat user strage: '%s'", path);
        return NULL;
    }
    encrypted = ap_palloc(r->pool, finfo.st_size);
    msg_r_len = read(fd, encrypted, finfo.st_size);
    ap_pclosef(r->pool, fd);
    msg = decrypt_message(r, encrypted, msg_r_len, key);

    return msg;
}


static int encrypt_store_handler(request_rec *r)
{
    ApacheRequest *req;
    const char *error_msg;

    r->content_type = "text/html; charset=EUC-JP";      
    ap_send_http_header(r);

    if (r->method_number != M_POST) {
        ap_rprintf(r, HTML_FORM, "", "");
        return OK;
    }
    req = ApacheRequest_new(r);
    /* ϥå */
    error_msg = do_input_check(r->pool, req);
    if (error_msg != NULL) {
        ap_rprintf(r, HTML_FORM, error_msg, "");
        return OK;
    }

    if (ApacheRequest_param(req, "save") != NULL) {
        /* å¸ */
        if (save_message(r, req))
            ap_rprintf(r, HTML_FORM, "å¸ޤ", "");
        else
            ap_rprintf(r, HTML_FORM, "å¸˼Ԥޤ", "");
    }
    else if (ApacheRequest_param(req, "load") != NULL) {
        /* åɤ߹߽ */
        char *message = load_message(r, req);
        if (message != NULL)
            ap_rprintf(r, HTML_FORM, "åɤ߹ߤޤ",
                       ap_escape_html(r->pool, message));
        else
            ap_rprintf(r, HTML_FORM, "åɤ߹ߤ˼Ԥޤ", "");
    }
    else {
        ap_rprintf(r, HTML_FORM, "ϤǤ", "");
    }
    return OK;
}


/* Dispatch list of content handlers */
static const handler_rec encrypt_store_handlers[] = { 
    { "encrypt_store", encrypt_store_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT encrypt_store_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    encrypt_store_handlers,/* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

