#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "libapreq/apache_cookie.h"

/* The sample content handler */
static int get_cookie_handler(request_rec *r)
{
    ApacheCookieJar *cookies;
    int i, num_cookie;

    r->content_type = "text/html";      

    cookies = ApacheCookie_parse(r, NULL);
    ap_send_http_header(r);

    ap_rprintf(r, "<html><body>\n<h1>%s</h1>\n", "Get HTTP Cookie(s)");
    num_cookie = ApacheCookieJarItems(cookies);
    for (i = 0; i < num_cookie; i++) {
        ApacheCookie *c = ApacheCookieJarFetch(cookies, i);
	int j;

	for (j = 0; j < ApacheCookieItems(c); j++) {
            ap_rprintf(r,
                       "<span id=\"name\">%s</span>"
                       "<span id=\"value\">%s</span>\n",
                       c->name, ApacheCookieFetch(c, j));
	}
    }

    ap_rputs("</body></html>", r);
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec get_cookie_handlers[] = { 
    { "get_cookie", get_cookie_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT get_cookie_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    get_cookie_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
#ifdef EAPI
   ,NULL,                  /* EAPI: add_module                    */
    NULL,                  /* EAPI: remove_module                 */
    NULL,                  /* EAPI: rewrite_command               */
    NULL                   /* EAPI: new_connection                */
#endif
};

