#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

static const char TARGET_TYPE[]     = "text/html";
static const int MAX_CONTENT_LENGTH = 1024 * 100;

/* ʸin_textδɤȽꤷ        */
/* "ISO-2022-JP" "Shift_JIS" "EUC-JP"֤ */
/* ȽǤʤNULL֤             */
static const char *guess_charset(pool *p, unsigned char *in_text)
{
    unsigned char *ptr;

    for (ptr = in_text; *ptr; ptr++) {
        if (*ptr == 0x1B && *(ptr+1) == '$')
            return ap_pstrdup(p, "ISO-2022-JP");
        if (*ptr <= 0x7F)
            continue;
        if (0x81 <= *ptr && *ptr <= 0x9F)
            return ap_pstrdup(p, "Shift_JIS");
        if (0xA1 <= *ptr && *ptr <= 0xDF)
            return ap_pstrdup(p, "EUC-JP");
        if (*(ptr+1) <= 0xA0)
            return ap_pstrdup(p, "Shift_JIS");
    }
    return NULL;
}

static int add_charset_by_guess(request_rec *r)
{
    FILE *fp;
    size_t length;
    unsigned char *in_text;
    const char *charset;

    if (r->main != NULL)  /* is sub_request */
        return DECLINED;
    /* "text/html"ʳcontent-typeΥꥯȤ̵ */
    if (r->content_type == NULL ||
        strncmp(r->content_type, TARGET_TYPE, sizeof(TARGET_TYPE)) != 0)
    {
        return DECLINED;
    }
    /* charsetꤵƤ̵ */
    if (ap_strcasestr(r->content_type, "charset=") != NULL)
        return DECLINED;
    if (r->finfo.st_size <= 0)
        return DECLINED;

    /* ꥯȤ줿եƤin_textسǼ */
    length = r->finfo.st_size;
    if (length > MAX_CONTENT_LENGTH)
        length = MAX_CONTENT_LENGTH;
    in_text = (unsigned char *)ap_palloc(r->pool, length + 1);
    fp = ap_pfopen(r->pool, r->filename, "r");
    if (fp == NULL)
        return DECLINED;
    if (fread(in_text, sizeof(unsigned char), length, fp) < 1) {
        ap_pfclose(r->pool, fp);
        return DECLINED;
    }
    ap_pfclose(r->pool, fp);
    in_text[length] = '\0';
 
    /* CharsetȽꤷContent-TypecharsetͿ */
    charset = guess_charset(r->pool, in_text);
    if (charset == NULL)
        return DECLINED;
    r->content_type = ap_pstrcat(r->pool, TARGET_TYPE,
                                 "; charset=", charset, NULL);
    return OK;
}

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT guess_charset_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    NULL,                  /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    add_charset_by_guess,  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

