#include "httpd.h"
#include "http_config.h"
#include "ap_config.h"
#include "http_log.h"

module log_header_module;

/* եopen˻Ѥ륪ץ */
static const int    OPEN_FLAGS = (O_WRONLY | O_APPEND | O_CREAT);
static const mode_t OPEN_MODE  = (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);

typedef struct {
    char *fname;
    int fd;
} log_header_conf;

/* ϤեΥѥ */
static const char *set_header_log(cmd_parms *parms, void *dummy, char *arg)
{
    log_header_conf *conf;

    conf = ap_get_module_config(parms->server->module_config,
                                &log_header_module);
    conf->fname = arg;
    return NULL;
}

static command_rec log_header_cmds[] = {
    { "HeaderLog", set_header_log, NULL, RSRC_CONF, TAKE1,
      "header log filename"},
    { NULL },
};

static void *create_log_header_conf(pool *p, server_rec *s)
{
    log_header_conf *conf;
    
    conf = (log_header_conf *)ap_palloc(p, sizeof(log_header_conf));
    conf->fname = "";
    conf->fd = -1;
    return (void *)conf;
}

/* Virtual hostȤΥopen */
static void init_log_header(server_rec *s, pool *p)
{
    
    for (; s; s = s->next) {
        log_header_conf *conf;
        char *fname;

        conf = ap_get_module_config(s->module_config, &log_header_module);
        fname = ap_server_root_relative(p, conf->fname);
        if (conf->fd > 0 || *conf->fname == '\0')
            continue;
        conf->fd = ap_popenf(p, fname, OPEN_FLAGS, OPEN_MODE);
        if (conf->fd < 0) {
            ap_log_error(APLOG_MARK, APLOG_ERR, s,
                         "could not open header log file %s.", fname);
            exit(1);
        }
    }
}

/* r->headers_in, r->headers_outʸѴ */
static char *header_to_string(pool *p, const table *t) 
{
    array_header *array = ap_table_elts(t);
    table_entry *elts;
    int array_len, i;
    char *str = "";

    elts = (table_entry *)array->elts;
    array_len = array->nelts;
    for (i = 0; i < array_len; ++i) {
        str = ap_pstrcat(p, str, elts[i].key, ": ", elts[i].val, "\n", NULL);
    }
    return str;	
}

static int header_log_transaction(request_rec *orig)
{
    log_header_conf *conf;
    request_rec *r;
    char *str, *in_header, *out_header;
    
    conf = ap_get_module_config(orig->server->module_config,
                                &log_header_module);
    if (conf->fd < 0 || *conf->fname == '\0') /* can't logging */
        return DECLINED;

    /* ǽŪʥꥯȤ򸡺 */
    for (r = orig; r->next; r = r->next)
        continue;
    /* Request/Response header˽ */
    in_header = header_to_string(orig->pool, orig->headers_in);
    out_header = header_to_string(orig->pool, r->headers_out);
    str = ap_pstrcat(orig->pool,
                     orig->the_request, "\n", in_header, "\n",
                     r->protocol, " ", r->status_line, "\n", out_header, "\n",
                     NULL);
    write(conf->fd, str, strlen(str));
    return OK;
}

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT log_header_module = {
    STANDARD_MODULE_STUFF, 
    init_log_header,         /* module initializer                  */
    NULL,                    /* create per-dir    config structures */
    NULL,                    /* merge  per-dir    config structures */
    create_log_header_conf,  /* create per-server config structures */
    NULL,                    /* merge  per-server config structures */
    log_header_cmds,         /* table of config file commands       */
    NULL,                    /* [#8] MIME-typed-dispatched handlers */
    NULL,                    /* [#1] URI to filename translation    */
    NULL,                    /* [#4] validate user id from request  */
    NULL,                    /* [#5] check if the user is ok _here_ */
    NULL,                    /* [#3] check access by host address   */
    NULL,                    /* [#6] determine MIME type            */
    NULL,                    /* [#7] pre-run fixups                 */
    header_log_transaction,  /* [#9] log a transaction              */
    NULL,                    /* [#2] header parser                  */
    NULL,                    /* child_init                          */
    NULL,                    /* child_exit                          */
    NULL                     /* [#0] post read-request              */
};

