#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include <mysql.h>

static const char DB_HOSTNAME[]       = "hostname";
static const char USER[]              = "username";
static const char PASS[]              = "password";
static const char DB_NAME[]           = "dbname";
static const unsigned int PORT_NUM    = 0;    /* use default port */
static const unsigned int CLIENT_FLAG = 0;


/* The sample content handler */
static int mysql_connect_handler(request_rec *r)
{
    MYSQL mysql, *connection;
    const char *message;

    mysql_init(&mysql);
    connection = mysql_real_connect(&mysql, DB_HOSTNAME, USER, PASS,
                                    DB_NAME, PORT_NUM, NULL,
                                    CLIENT_FLAG);
    if (connection == NULL) {
        message = ap_pstrcat(r->pool,
                             "Failed to connect to database: ",
                             mysql_error(&mysql),
                             NULL);
    }
    else {
        message = "connect to database";
    }
    mysql_close(connection);
   

    r->content_type = "text/html";      
    ap_send_http_header(r);
    if (!r->header_only)
        ap_rprintf(r, "<h1>MySQL connect test</h1>%s", message);
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec mysql_connect_handlers[] = { 
    { "mysql_connect", mysql_connect_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT mysql_connect_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    mysql_connect_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

