#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include <http_log.h>
#include <mysql.h>

static const char DB_HOSTNAME[]       = "hostname";
static const char USER[]              = "username";
static const char PASS[]              = "password";
static const char DB_NAME[]           = "dbname";
static const unsigned int PORT_NUM    = 0;    /* use default port */
static const unsigned int CLIENT_FLAG = 0;

static MYSQL mysql, *connection = NULL;

enum {
    CONNECT_UNSUCCESSFUL = 0,
    CONNECT_SUCCESSFUL   = 1,
    CONNECT_ALREADY      = 2,
};


static int connect_db()
{
    if (connection != NULL && mysql_ping(connection) == 0)
        return CONNECT_ALREADY;

    mysql_init(&mysql);
    connection = mysql_real_connect(&mysql, DB_HOSTNAME, USER, PASS,
                               DB_NAME, PORT_NUM, NULL, CLIENT_FLAG);
    if (connection == NULL) {
        return CONNECT_UNSUCCESSFUL;
    }
    return CONNECT_SUCCESSFUL;
}

static void child_init(server_rec *s, pool *p)
{
    (void)connect_db();
}

static void child_exit(server_rec *s, pool *p)
{
    mysql_close(&mysql);
}

static int mysql_keep_connect_handler(request_rec *r)
{
    int status;
    const char *message;

    status = connect_db();
    if (status == CONNECT_UNSUCCESSFUL) {
        message = ap_pstrcat(r->pool,
                             "Failed to connect to database: ",
                             mysql_error(&mysql),
                             NULL);
    }
    else if (status == CONNECT_SUCCESSFUL){
        message = ap_pstrdup(r->pool, "connection successful");
    }
    else if (status == CONNECT_ALREADY) {
        message = ap_pstrdup(r->pool, "connection already");
    }
    else {
        message = ap_pstrdup(r->pool, "unknown status");
    }

    r->content_type = "text/html";      
    ap_send_http_header(r);
    if (!r->header_only)
        ap_rprintf(r, "<h1>MySQL connect test</h1>%s", message);
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec mysql_keep_connect_handlers[] = { 
    { "mysql_keep_connect", mysql_keep_connect_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT mysql_keep_connect_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    mysql_keep_connect_handlers,    /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    child_init,            /* child_init                          */
    child_exit,            /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

