#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

static table *get_param_by_urlencoded(pool *p, char *src)
{
    table *param;
    char *token;
    char *args = src;

    if (args == NULL)
        return NULL;
    param = ap_make_table(p, 1);
    while (*args && (token = ap_getword(p, &args, '&'))) {
        char *key, *value;
        key   = ap_getword(p, &token, '=');
        value = ap_getword(p, &token, '=');
        ap_unescape_url(key);
        ap_unescape_url(value);
        ap_table_set(param, key, value);
    }
    return param;
}

static table *get_param_by_body(request_rec *r)
{
    char *body;
    char buff[HUGE_STRING_LEN];
    int rsize, len_read;
    int rpos = 0;
    long length;

    if (ap_setup_client_block(r, REQUEST_CHUNKED_ERROR))
        return NULL;
    if (!ap_should_client_block(r))
        return NULL;

    length = r->remaining;
    body = ap_pcalloc(r->pool, length + 1);
    while ((len_read = ap_get_client_block(r, buff, sizeof(buff))) > 0) {
        if ((rpos + len_read) > length)
	    rsize = length - rpos;
	else
	    rsize = len_read;
	memmove(body + rpos, buff, rsize);
	rpos += rsize;
    }
    return get_param_by_urlencoded(r->pool, body);
}

static int parse_param_handler(request_rec *r)
{
    table *param;

    r->content_type = "text/plain";      
    ap_send_http_header(r);
    if (r->header_only)
        return OK;

    if (r->method_number == M_GET)
        param = get_param_by_urlencoded(r->pool, r->args);
    else if (r->method_number == M_POST)
        param = get_param_by_body(r);
    if (param == NULL) {
        ap_rputs("ѥ᡼Ϥޤ", r);
        return OK;
    }
    ap_rprintf(r, "̾: %s\n", ap_table_get(param, "name"));
    ap_rprintf(r, "ǯ: %s\n", ap_table_get(param, "age"));

    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec parse_param_handlers[] = { 
    { "parse_param", parse_param_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT parse_param_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    parse_param_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

