#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

#include <libpq-fe.h>

static const char CONNECT_PARAM[] =
	"host=hostname "
	"dbname=dbname "
	"user=username "
	"password=password ";

static int pg_connect_handler(request_rec *r)
{
    PGconn *conn;         /* PostgreSQL³ϥɥ */
    const char *message;  /* ³ɽѥå */

    /* PostgreSQLФ³³ϥɥ */
    conn = PQconnectdb(CONNECT_PARAM);
    if (PQstatus(conn) == CONNECTION_BAD) {
        message = ap_pstrcat(r->pool,
                             "Failed to connect to database: ",
                             PQerrorMessage(conn),
			     NULL);
    } else {
        message = "connect to database";
    }
    PQfinish(conn);

    r->content_type = "text/html";      
    ap_send_http_header(r);
    if (!r->header_only)
       ap_rprintf(r, "<h1>PostgreSQL connect test</h1>%s", message);
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec pg_connect_handlers[] = { 
    { "pg_connect", pg_connect_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT pg_connect_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    pg_connect_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

