#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

#include <libpq-fe.h>

static const char CONNECT_PARAM[] =
    "host=hostname "
    "dbname=dbname "
    "user=username "
    "password=password ";

static PGconn *conn = NULL;   /* PostgreSQL³ϥɥ */

enum {
    CONNECT_UNSUCCESSFUL = 0,
    CONNECT_SUCCESSFUL   = 1,
    CONNECT_ALREADY      = 2,
};


static int ping_db(void)
{
    PGresult *result;
    int status;

    if (conn == NULL || PQstatus(conn) == CONNECTION_BAD)
        return CONNECT_UNSUCCESSFUL;

    result = PQexec(conn, " ");
    status = result ? PQresultStatus(result) : -1;
    PQclear(result);

    if (status != PGRES_EMPTY_QUERY)
        return CONNECT_UNSUCCESSFUL;

    return CONNECT_SUCCESSFUL;
}


static int connect_db(void)
{
    if (ping_db() == CONNECT_SUCCESSFUL)
        return CONNECT_ALREADY;

    conn = PQconnectdb(CONNECT_PARAM);
    if (PQstatus(conn) == CONNECTION_BAD)
        return CONNECT_UNSUCCESSFUL;
    return CONNECT_SUCCESSFUL;
}

static void child_init(server_rec *s, pool *p)
{
    (void)connect_db();
}

static void child_exit(server_rec *s, pool *p)
{
    if (conn != NULL)
        PQfinish(conn);
}


static int pg_keep_connect_handler(request_rec *r)
{
    int status;           /* ³ϥɥΥơ */
    const char *message;  /* ³ɽѥå */

    status = connect_db();
    if (status == CONNECT_UNSUCCESSFUL) {
        message = ap_pstrcat(r->pool,
                             "Failed to connect to database: ",
                             PQerrorMessage(conn),
                             NULL);
    }
    else if (status == CONNECT_SUCCESSFUL) {
        message = ap_pstrdup(r->pool, "connection successful");
    }
    else if (status == CONNECT_ALREADY) {
        message = ap_pstrdup(r->pool, "connection already");
    }
    else {
        message = ap_pstrdup(r->pool, "unknown status");
    }

    r->content_type = "text/html";      
    ap_send_http_header(r);
    if (!r->header_only)
       ap_rprintf(r, "<h1>PostgreSQL connect test</h1>%s", message);
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec pg_keep_connect_handlers[] = { 
    { "pg_keep_connect", pg_keep_connect_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT pg_keep_connect_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    pg_keep_connect_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    child_init,            /* child_init                          */
    child_exit,            /* child_exit                          */
    NULL                   /* [#0] post read-request              */
};

