#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"
#include "http_log.h"

const char ETC_SERVICES[] = "/etc/services";

/* The sample content handler */
static int sendfile_handler(request_rec *r)
{
    FILE *file;
    struct stat finfo;

    if ((stat(ETC_SERVICES, &finfo) == -1) || finfo.st_mode == 0) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
	              "File does not exist: %s", ETC_SERVICES);
        return HTTP_INTERNAL_SERVER_ERROR;
    }
    file = ap_pfopen(r->pool, ETC_SERVICES, "r");
    if (file == NULL) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
                      "file permissions deny server access: %s", ETC_SERVICES);
        return HTTP_FORBIDDEN;
    }

    r->content_type = "text/plain";
    ap_send_http_header(r);
    if (r->header_only)
        return OK;

    ap_send_fd(file, r);
    ap_pfclose(r->pool, file);
    return OK;
}

static const handler_rec sendfile_handlers[] = { 
    { "sendfile", sendfile_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT sendfile_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    sendfile_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
#ifdef EAPI
   ,NULL,                  /* EAPI: add_module                    */
    NULL,                  /* EAPI: remove_module                 */
    NULL,                  /* EAPI: rewrite_command               */
    NULL                   /* EAPI: new_connection                */
#endif
};

